﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Globalization;

class Program {
	static void Main(string[] args) {
		string s1 = "Řáa\u0301\U0001F0B9";
		Console.WriteLine("*** Using string: \"Řáa\\u0301\\U0001F0B9\"");

		byte[] ba = Encoding.UTF8.GetBytes(s1);
		PrintBytes(ba, "Utf8   ");

		ba = Encoding.Unicode.GetBytes(s1);
		PrintBytes(ba, "Utf16  ");

		ba = Encoding.BigEndianUnicode.GetBytes(s1);
		PrintBytes(ba, "Utf16BE");

		ba = Encoding.UTF32.GetBytes(s1);
		PrintBytes(ba, "Utf32  ");

		Console.Write("Press ENTER to show more examples ...");
		Console.ReadLine();
		Console.WriteLine();

		s1 = "Řáa\u0301\U0001F0B9\U0001D4D0\u0301";
		Console.WriteLine("*** Using string: \"Řáa\\u0301\\U0001F0B9\\U0001D4D0\\u0301\"");

		ba = Encoding.ASCII.GetBytes(s1);
		PrintBytes(ba, "ASCII  ");

		ba = Encoding.GetEncoding(1250).GetBytes(s1);
		PrintBytes(ba, "Win1250");
		ShowInNotepad(ba, "Win1250");

		ba = Encoding.Unicode.GetBytes(s1);
		PrintBytes(ba, "Utf16  ");
		ShowInNotepad(ba, "Utf16");

		Console.Write("Press ENTER to show more examples ...");
		Console.ReadLine();
		Console.WriteLine();

		ba = Encoding.GetEncoding(1252).GetBytes(s1);
		PrintBytes(ba, "Win1252");

		ba = Encoding.GetEncoding(1252, EncoderFallback.ReplacementFallback, DecoderFallback.ReplacementFallback).GetBytes(s1);
		PrintBytes(ba, "Win1252");

		ba = Encoding.GetEncoding(1252, new EncoderReplacementFallback("<>"), DecoderFallback.ReplacementFallback).GetBytes(s1);
		PrintBytes(ba, "Win1252");

		try {
			ba = Encoding.GetEncoding(1252, EncoderFallback.ExceptionFallback, DecoderFallback.ExceptionFallback).GetBytes(s1);
			PrintBytes(ba, "Win1252");
		} catch (Exception ex) {
			Console.WriteLine("Exception '{0}' caught:", ex.GetType());
			Console.WriteLine(ex);

			var fallbackEx = ex as System.Text.EncoderFallbackException;
			if (fallbackEx != null) {
				Console.WriteLine("System.Text.EncoderFallbackException:");
				Console.WriteLine("\t.CharUnknown: '{0}' (0x{1:X4})", fallbackEx.CharUnknown, (int) fallbackEx.CharUnknown);
				Console.WriteLine("\t.CharUnknownHigh: '{0}' (0x{1:X4})", fallbackEx.CharUnknownHigh, (int) fallbackEx.CharUnknownHigh);
				Console.WriteLine("\t.CharUnknownLow: '{0}' (0x{1:X4})", fallbackEx.CharUnknownLow, (int) fallbackEx.CharUnknownLow);
				Console.WriteLine("\t.Index: {0}", fallbackEx.Index);
			}
		}

		Console.Write("Press ENTER to show more examples ...");
		Console.ReadLine();
		Console.WriteLine();

		Console.WriteLine("new System.IO.StreamReader(\"Utf16.txt\")");
		using (var reader = new System.IO.StreamReader("Utf16.txt")) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		Console.WriteLine("new System.IO.StreamReader(\"Utf16.txt\", detectEncodingFromByteOrderMarks: true)");
		using (var reader = new System.IO.StreamReader("Utf16.txt", detectEncodingFromByteOrderMarks: true)) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		Console.WriteLine("new System.IO.StreamReader(\"Utf16.txt\", Encoding.Unicode)");
		using (var reader = new System.IO.StreamReader("Utf16.txt", Encoding.Unicode)) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		Console.WriteLine("new System.IO.StreamReader(\"Utf16.txt\", Encoding.GetEncoding(1250))");
		using (var reader = new System.IO.StreamReader("Utf16.txt", Encoding.GetEncoding(1250))) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		Console.Write("Press ENTER to show more examples ...");
		Console.ReadLine();
		Console.WriteLine();

		using (var writer = new System.IO.StreamWriter("WriterUtf16.txt", append: false, encoding: Encoding.Unicode)) {
			writer.WriteLine(s1);
		}

		Console.WriteLine("new System.IO.StreamReader(\"WriterUtf16.txt\", detectEncodingFromByteOrderMarks: true)");
		using (var reader = new System.IO.StreamReader("WriterUtf16.txt", detectEncodingFromByteOrderMarks: true)) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		using (var writer = new System.IO.StreamWriter("WriterUtf16.txt", append: false, encoding: new UnicodeEncoding(bigEndian: false, byteOrderMark: false))) {
			writer.WriteLine(s1);
		}

		Console.WriteLine("new System.IO.StreamReader(\"WriterUtf16.txt\", detectEncodingFromByteOrderMarks: true)");
		using (var reader = new System.IO.StreamReader("WriterUtf16.txt", detectEncodingFromByteOrderMarks: true)) {
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
			Console.WriteLine("\tLine read: '{0}'", reader.ReadLine());
			Console.WriteLine("\t{0}", reader.CurrentEncoding);
		}

		/**/
	}

	static void PrintBytes(byte[] s, string encodingName) {
		Console.Write("# {0} bytes: ", encodingName);
		foreach (byte b in s) {
			Console.Write("{0:X2}({1}) ", b, (char) b);
		}
		Console.WriteLine();
	}

	static void ShowInNotepad(byte[] s, string encodingName) {
		string fileName = encodingName + ".txt";
		using (var fs = new System.IO.FileStream(fileName, System.IO.FileMode.Create)) {
			fs.Write(s, 0, s.Length);
		}

		System.Diagnostics.Process.Start("notepad.exe", fileName);
	}
}
